@@ -1,19 +1,19 @@
 #  File R/startup.utilities.R in package statnet.common, part of the Statnet suite
-#  of packages for network analysis, http://statnet.org .
+#  of packages for network analysis, https://statnet.org .
 #
 #  This software is distributed under the GPL-3 license.  It is free,
 #  open source, and has the attribution requirements (GPL Section 7) at
-#  http://statnet.org/attribution
+#  https://statnet.org/attribution
 #
 #  Copyright 2007-2019 Statnet Commons
 #######################################################################
 ## .who.loaded.me <- function(){
-##   top.call <- sys.calls()[[1]] # Grab the top-level call.
-##   top.fn <- as.character(top.call[[1]])
+##   top.call <- sys.calls()[[1L]] # Grab the top-level call.
+##   top.fn <- as.character(top.call[[1L]])
   
 ##   if(length(top.fn)!=1 || !(top.fn %in% c("library","require"))) return(NULL)
 
-##   top.call <- match.call(get(as.character(top.call[[1]]),baseenv(),mode="function"),top.call) # Expand the arguments.
+##   top.call <- match.call(get(as.character(top.call[[1L]]),baseenv(),mode="function"),top.call) # Expand the arguments.
 ##   top.call <- as.list(top.call) # Turn the call into a list.
   
 ##   top.pkg <- top.call$package
@@ -83,7 +83,10 @@ statnetStartupMessage <- function(pkgname, friends, nofriends){
                    ucla.edu="University of California -- Los Angeles",
                    nyu.edu="New York University",
                    murdoch.edu.au="Murdoch University",
-                   uow.edu.au="University of Wollongong"
+                   uow.edu.au="University of Wollongong",
+                   unsw.edu.au="University of New South Wales",
+                   kozminski.edu.pl="Kozminski University",
+                   uw.edu.pl="University of Warsaw"
                    ) 
 
   # Note that all options are ignored at this time, and the "wall of
@@ -114,7 +117,7 @@ statnetStartupMessage <- function(pkgname, friends, nofriends){
   pnlines <- ifelse(pninsts=="", pnnames, paste(pnnames,pninsts, sep=", "))
   
   copylist <- paste("Copyright (c) ",substr(desc$Date,1,4),", ",sep="")
-  copylist <- paste(copylist, pnlines[authors][1],"\n",
+  copylist <- paste(copylist, pnlines[authors][1L],"\n",
                     paste(
                       paste(rep(" ",nchar(copylist)),collapse=""),
                       c(pnlines[authors][-1],if(sum(!authors)) "with contributions from",pnlines[!authors]),sep="",collapse="\n"),
